/*
 \verbatim

    This file is part of CSSEngine. CSSEng is a CSS (Cascaded Style Sheet)
    parsing and selecting engine derived from libcss of NetSurf project.

    Licensed under the MIT License,

           http://www.opensource.org/licenses/mit-license.php

    Copyright 2008 John-Mark Bell <jmb@netsurf-browser.org>
    Copyright (C) 2021 ~ 2023 Beijing FMSoft Technologies Co., Ltd.

 \endverbatim
 */

/*
 * This file was generated by CSSEng gen_parser 
 * 
 * Generated from:
 *
 * appearance:CSS_PROP_APPEARANCE IDENT:( INHERIT: NONE:0,APPEARANCE_NONE AUTO:0,APPEARANCE_AUTO MENULIST:0,APPEARANCE_MENULIST MENULIST_BUTTON:0,APPEARANCE_MENULIST_BUTTON TEXTFIELD:0,APPEARANCE_TEXTFIELD TEXTAREA:0,APPEARANCE_TEXTAREA PROGRESS_BAR:0,APPEARANCE_PROGRESS_BAR PROGRESS_BKGND:0,APPEARANCE_PROGRESS_BKGND PROGRESS_MARK:0,APPEARANCE_PROGRESS_MARK METER:0,APPEARANCE_METER METER_BAR:0,APPEARANCE_METER_BAR METER_BKGND:0,APPEARANCE_METER_BKGND METER_MARK:0,APPEARANCE_METER_MARK SLIDER_HORIZONTAL:0,APPEARANCE_SLIDER_HORIZONTAL SLIDER_VERTICAL:0,APPEARANCE_SLIDER_VERTICAL BUTTON:0,APPEARANCE_BUTTON CHECKBOX:0,APPEARANCE_CHECKBOX LISTBOX:0,APPEARANCE_LISTBOX RADIO:0,APPEARANCE_RADIO SEARCHFIELD:0,APPEARANCE_SEARCHFIELD PUSH_BUTTON:0,APPEARANCE_PUSH_BUTTON SQUARE_BUTTON:0,APPEARANCE_SQUARE_BUTTON IDENT:)
 * 
 * Copyright (C) 2021, 2023 Beijing FMSoft Technologies Co., Ltd.
 */

#include <assert.h>
#include <string.h>

#include "bytecode/bytecode.h"
#include "bytecode/opcodes.h"
#include "parse/properties/properties.h"
#include "parse/properties/utils.h"

/**
 * Parse appearance
 *
 * \param c      Parsing context
 * \param vector  Vector of tokens to process
 * \param ctx      Pointer to vector iteration context
 * \param result  resulting style
 * \return CSS_OK on success,
 *       CSS_NOMEM on memory exhaustion,
 *       CSS_INVALID if the input is not valid
 *
 * Post condition: \a *ctx is updated with the next token to process
 *           If the input is invalid, then \a *ctx remains unchanged.
 */
css_error css__parse_appearance(css_language *c,
        const parserutils_vector *vector, int *ctx,
        css_style *result)
{
    int orig_ctx = *ctx;
    css_error error;
    const css_token *token;
    bool match;

    token = parserutils_vector_iterate(vector, ctx);
    if ((token == NULL) || ((token->type != CSS_TOKEN_IDENT))) {
		*ctx = orig_ctx;
		return CSS_INVALID;
	}

	if ((lwc_string_caseless_isequal(token->idata, c->strings[INHERIT], &match) == lwc_error_ok && match)) {
			error = css_stylesheet_style_inherit(result, CSS_PROP_APPEARANCE);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[NONE], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_NONE);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[AUTO], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_AUTO);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[MENULIST], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_MENULIST);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[MENULIST_BUTTON], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_MENULIST_BUTTON);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[TEXTFIELD], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_TEXTFIELD);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[TEXTAREA], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_TEXTAREA);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[PROGRESS_BAR], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_PROGRESS_BAR);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[PROGRESS_BKGND], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_PROGRESS_BKGND);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[PROGRESS_MARK], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_PROGRESS_MARK);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[METER], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_METER);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[METER_BAR], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_METER_BAR);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[METER_BKGND], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_METER_BKGND);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[METER_MARK], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_METER_MARK);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SLIDER_HORIZONTAL], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_SLIDER_HORIZONTAL);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SLIDER_VERTICAL], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_SLIDER_VERTICAL);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[BUTTON], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_BUTTON);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[CHECKBOX], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_CHECKBOX);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[LISTBOX], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_LISTBOX);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[RADIO], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_RADIO);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SEARCHFIELD], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_SEARCHFIELD);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[PUSH_BUTTON], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_PUSH_BUTTON);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SQUARE_BUTTON], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_APPEARANCE, 0,APPEARANCE_SQUARE_BUTTON);
	} else {
		error = CSS_INVALID;
	}

    if (error != CSS_OK)
        *ctx = orig_ctx;
    
    return error;
}

