/*
 \verbatim

    This file is part of CSSEngine. CSSEng is a CSS (Cascaded Style Sheet)
    parsing and selecting engine derived from libcss of NetSurf project.

    Licensed under the MIT License,

           http://www.opensource.org/licenses/mit-license.php

    Copyright 2008 John-Mark Bell <jmb@netsurf-browser.org>
    Copyright (C) 2021 ~ 2023 Beijing FMSoft Technologies Co., Ltd.

 \endverbatim
 */

/*
 * This file was generated by CSSEng gen_parser 
 * 
 * Generated from:
 *
 * comp_op:CSS_PROP_COMP_OP IDENT:( INHERIT: CLEAR:0,COMP_OP_CLEAR SRC:0,COMP_OP_SRC DST:0,COMP_OP_DST SRC_OVER:0,COMP_OP_SRC_OVER DST_OVER:0,COMP_OP_DST_OVER SRC_IN:0,COMP_OP_SRC_IN DST_IN:0,COMP_OP_DST_IN SRC_OUT:0,COMP_OP_SRC_OUT DST_OUT:0,COMP_OP_DST_OUT SRC_ATOP:0,COMP_OP_SRC_ATOP DST_ATOP:0,COMP_OP_DST_ATOP XOR:0,COMP_OP_XOR PLUS:0,COMP_OP_PLUS MULTIPLY:0,COMP_OP_MULTIPLY SCREEN:0,COMP_OP_SCREEN OVERLAY:0,COMP_OP_OVERLAY DARKEN:0,COMP_OP_DARKEN LIGHTEN:0,COMP_OP_LIGHTEN COLOR_DODGE:0,COMP_OP_COLOR_DODGE COLOR_BURN:0,COMP_OP_COLOR_BURN HARD_LIGHT:0,COMP_OP_HARD_LIGHT SOFT_LIGHT:0,COMP_OP_SOFT_LIGHT DIFFERENCE:0,COMP_OP_DIFFERENCE EXCLUSION:0,COMP_OP_EXCLUSION IDENT:)
 * 
 * Copyright (C) 2021, 2023 Beijing FMSoft Technologies Co., Ltd.
 */

#include <assert.h>
#include <string.h>

#include "bytecode/bytecode.h"
#include "bytecode/opcodes.h"
#include "parse/properties/properties.h"
#include "parse/properties/utils.h"

/**
 * Parse comp_op
 *
 * \param c      Parsing context
 * \param vector  Vector of tokens to process
 * \param ctx      Pointer to vector iteration context
 * \param result  resulting style
 * \return CSS_OK on success,
 *       CSS_NOMEM on memory exhaustion,
 *       CSS_INVALID if the input is not valid
 *
 * Post condition: \a *ctx is updated with the next token to process
 *           If the input is invalid, then \a *ctx remains unchanged.
 */
css_error css__parse_comp_op(css_language *c,
        const parserutils_vector *vector, int *ctx,
        css_style *result)
{
    int orig_ctx = *ctx;
    css_error error;
    const css_token *token;
    bool match;

    token = parserutils_vector_iterate(vector, ctx);
    if ((token == NULL) || ((token->type != CSS_TOKEN_IDENT))) {
		*ctx = orig_ctx;
		return CSS_INVALID;
	}

	if ((lwc_string_caseless_isequal(token->idata, c->strings[INHERIT], &match) == lwc_error_ok && match)) {
			error = css_stylesheet_style_inherit(result, CSS_PROP_COMP_OP);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[CLEAR], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_CLEAR);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SRC], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SRC);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DST], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DST);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SRC_OVER], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SRC_OVER);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DST_OVER], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DST_OVER);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SRC_IN], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SRC_IN);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DST_IN], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DST_IN);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SRC_OUT], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SRC_OUT);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DST_OUT], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DST_OUT);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SRC_ATOP], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SRC_ATOP);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DST_ATOP], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DST_ATOP);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[XOR], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_XOR);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[PLUS], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_PLUS);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[MULTIPLY], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_MULTIPLY);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SCREEN], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SCREEN);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[OVERLAY], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_OVERLAY);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DARKEN], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DARKEN);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[LIGHTEN], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_LIGHTEN);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[COLOR_DODGE], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_COLOR_DODGE);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[COLOR_BURN], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_COLOR_BURN);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[HARD_LIGHT], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_HARD_LIGHT);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[SOFT_LIGHT], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_SOFT_LIGHT);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[DIFFERENCE], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_DIFFERENCE);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[EXCLUSION], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_COMP_OP, 0,COMP_OP_EXCLUSION);
	} else {
		error = CSS_INVALID;
	}

    if (error != CSS_OK)
        *ctx = orig_ctx;
    
    return error;
}

