/*
 \verbatim

    This file is part of CSSEngine. CSSEng is a CSS (Cascaded Style Sheet)
    parsing and selecting engine derived from libcss of NetSurf project.

    Licensed under the MIT License,

           http://www.opensource.org/licenses/mit-license.php

    Copyright 2008 John-Mark Bell <jmb@netsurf-browser.org>
    Copyright (C) 2021 ~ 2023 Beijing FMSoft Technologies Co., Ltd.

 \endverbatim
 */

/*
 * This file was generated by CSSEng gen_parser 
 * 
 * Generated from:
 *
 * text_anchor:CSS_PROP_TEXT_ANCHOR IDENT:( INHERIT: START:0,TEXT_ANCHOR_START MIDDLE:0,TEXT_ANCHOR_MIDDLE END:0,TEXT_ANCHOR_END IDENT:)
 * 
 * Copyright (C) 2021, 2023 Beijing FMSoft Technologies Co., Ltd.
 */

#include <assert.h>
#include <string.h>

#include "bytecode/bytecode.h"
#include "bytecode/opcodes.h"
#include "parse/properties/properties.h"
#include "parse/properties/utils.h"

/**
 * Parse text_anchor
 *
 * \param c      Parsing context
 * \param vector  Vector of tokens to process
 * \param ctx      Pointer to vector iteration context
 * \param result  resulting style
 * \return CSS_OK on success,
 *       CSS_NOMEM on memory exhaustion,
 *       CSS_INVALID if the input is not valid
 *
 * Post condition: \a *ctx is updated with the next token to process
 *           If the input is invalid, then \a *ctx remains unchanged.
 */
css_error css__parse_text_anchor(css_language *c,
        const parserutils_vector *vector, int *ctx,
        css_style *result)
{
    int orig_ctx = *ctx;
    css_error error;
    const css_token *token;
    bool match;

    token = parserutils_vector_iterate(vector, ctx);
    if ((token == NULL) || ((token->type != CSS_TOKEN_IDENT))) {
		*ctx = orig_ctx;
		return CSS_INVALID;
	}

	if ((lwc_string_caseless_isequal(token->idata, c->strings[INHERIT], &match) == lwc_error_ok && match)) {
			error = css_stylesheet_style_inherit(result, CSS_PROP_TEXT_ANCHOR);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[START], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_TEXT_ANCHOR, 0,TEXT_ANCHOR_START);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[MIDDLE], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_TEXT_ANCHOR, 0,TEXT_ANCHOR_MIDDLE);
	} else if ((lwc_string_caseless_isequal(token->idata, c->strings[END], &match) == lwc_error_ok && match)) {
			error = css__stylesheet_style_appendOPV(result, CSS_PROP_TEXT_ANCHOR, 0,TEXT_ANCHOR_END);
	} else {
		error = CSS_INVALID;
	}

    if (error != CSS_OK)
        *ctx = orig_ctx;
    
    return error;
}

